@@ -41,3 +41,14 @@ g_io_module_unload (GIOModule   *module)
   _fam_sub_shutdown ();
 }
 
+char **
+g_io_module_query (void)
+{
+  char *eps[] = {
+    G_LOCAL_DIRECTORY_MONITOR_EXTENSION_POINT_NAME,
+    G_LOCAL_FILE_MONITOR_EXTENSION_POINT_NAME,
+    NULL
+  };
+  return g_strdupv (eps);
+}
+
