@@ -148,6 +148,7 @@
  * <row><entry>%G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE</entry><entry>standard::content-type</entry><entry>string</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE</entry><entry>standard::fast-content-type</entry><entry>string</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_STANDARD_SIZE</entry><entry>standard::size</entry><entry>uint64</entry></row>
+ * <row><entry>%G_FILE_ATTRIBUTE_STANDARD_ALLOCATED_SIZE</entry><entry>standard::allocated-size</entry><entry>uint64</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET</entry><entry>standard::symlink-target</entry><entry>byte string</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_STANDARD_TARGET_URI</entry><entry>standard::target-uri</entry><entry>string</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_STANDARD_SORT_ORDER</entry><entry>standard::sort-order</entry><entry>int32</entry></row>
@@ -164,6 +165,7 @@
  * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_CAN_UNMOUNT</entry><entry>mountable::can-unmount</entry><entry>boolean</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_CAN_EJECT</entry><entry>mountable::can-eject</entry><entry>boolean</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE</entry><entry>mountable::unix-device</entry><entry>uint32</entry></row>
+ * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE_FILE</entry><entry>mountable::unix-device-file</entry><entry>string</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_HAL_UDI</entry><entry>mountable::hal-udi</entry><entry>string</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_TIME_MODIFIED</entry><entry>time::modified</entry><entry>uint64</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC</entry><entry>time::modified-usec</entry><entry>uint32</entry></row>
@@ -242,6 +244,9 @@ _g_file_attribute_value_clear (GFileAttributeValue *attr)
       attr->type == G_FILE_ATTRIBUTE_TYPE_BYTE_STRING)
     g_free (attr->u.string);
   
+  if (attr->type == G_FILE_ATTRIBUTE_TYPE_STRINGV)
+    g_strfreev (attr->u.stringv);
+
   if (attr->type == G_FILE_ATTRIBUTE_TYPE_OBJECT &&
       attr->u.obj != NULL)
     g_object_unref (attr->u.obj);
@@ -270,6 +275,9 @@ _g_file_attribute_value_set (GFileAttributeValue        *attr,
       attr->type == G_FILE_ATTRIBUTE_TYPE_BYTE_STRING)
     attr->u.string = g_strdup (attr->u.string);
   
+  if (attr->type == G_FILE_ATTRIBUTE_TYPE_STRINGV)
+    attr->u.stringv = g_strdupv (attr->u.stringv);
+
   if (attr->type == G_FILE_ATTRIBUTE_TYPE_OBJECT &&
       attr->u.obj != NULL)
     g_object_ref (attr->u.obj);
@@ -299,6 +307,8 @@ _g_file_attribute_value_peek_as_pointer (GFileAttributeValue *attr)
   case G_FILE_ATTRIBUTE_TYPE_STRING:
   case G_FILE_ATTRIBUTE_TYPE_BYTE_STRING:
     return attr->u.string;
+  case G_FILE_ATTRIBUTE_TYPE_STRINGV:
+    return attr->u.stringv;
   case G_FILE_ATTRIBUTE_TYPE_OBJECT:
     return attr->u.obj;
   default:
@@ -358,17 +368,17 @@ escape_byte_string (const char *str)
   int num_invalid, i;
   char *escaped_val, *p;
   unsigned char c;
-  char *hex_digits = "0123456789abcdef";
-  
+  const char hex_digits[] = "0123456789abcdef";
+
   len = strlen (str);
-  
+
   num_invalid = 0;
   for (i = 0; i < len; i++)
     {
       if (!valid_char (str[i]))
 	num_invalid++;
     }
-	
+
   if (num_invalid == 0)
     return g_strdup (str);
   else
@@ -395,18 +405,20 @@ escape_byte_string (const char *str)
 }
 
 /*
- * g_file_attribute_value_as_string:
+ * _g_file_attribute_value_as_string:
  * @attr: a #GFileAttributeValue.
  *
  * Converts a #GFileAttributeValue to a string for display.
  * The returned string should be freed when no longer needed.
- * 
- * Returns: a string from the @attr, %NULL on error, or "&lt;invalid&gt;" 
+ *
+ * Returns: a string from the @attr, %NULL on error, or "&lt;invalid&gt;"
  * if @attr is of type %G_FILE_ATTRIBUTE_TYPE_INVALID.
- **/
+ */
 char *
 _g_file_attribute_value_as_string (const GFileAttributeValue *attr)
 {
+  GString *s;
+  int i;
   char *str;
 
   g_return_val_if_fail (attr != NULL, NULL);
@@ -416,6 +428,17 @@ _g_file_attribute_value_as_string (const GFileAttributeValue *attr)
     case G_FILE_ATTRIBUTE_TYPE_STRING:
       str = g_strdup (attr->u.string);
       break;
+    case G_FILE_ATTRIBUTE_TYPE_STRINGV:
+      s = g_string_new ("[");
+      for (i = 0; attr->u.stringv[i] != NULL; i++)
+	{
+	  g_string_append (s, attr->u.stringv[i]);
+	  if (attr->u.stringv[i+1] != NULL)
+	    g_string_append (s, ", ");
+	}
+      g_string_append (s, "]");
+      str = g_string_free (s, FALSE);
+      break;
     case G_FILE_ATTRIBUTE_TYPE_BYTE_STRING:
       str = escape_byte_string (attr->u.string);
       break;
@@ -439,24 +462,27 @@ _g_file_attribute_value_as_string (const GFileAttributeValue *attr)
                                           ((GTypeInstance *) attr->u.obj),
                                       attr->u.obj);
       break;
+    case G_FILE_ATTRIBUTE_TYPE_INVALID:
+      str = g_strdup ("<unset>");
+      break;
     default:
       g_warning ("Invalid type in GFileInfo attribute");
       str = g_strdup ("<invalid>");
       break;
     }
-  
+
   return str;
 }
 
 /*
- * g_file_attribute_value_get_string:
+ * _g_file_attribute_value_get_string:
  * @attr: a #GFileAttributeValue.
- * 
+ *
  * Gets the string from a file attribute value. If the value is not the
  * right type then %NULL will be returned.
- * 
+ *
  * Returns: the string value contained within the attribute, or %NULL.
- **/
+ */
 const char *
 _g_file_attribute_value_get_string (const GFileAttributeValue *attr)
 {
@@ -469,14 +495,14 @@ _g_file_attribute_value_get_string (const GFileAttributeValue *attr)
 }
 
 /*
- * g_file_attribute_value_get_byte_string:
+ * _g_file_attribute_value_get_byte_string:
  * @attr: a #GFileAttributeValue.
- * 
+ *
  * Gets the byte string from a file attribute value. If the value is not the
  * right type then %NULL will be returned.
- * 
+ *
  * Returns: the byte string contained within the attribute or %NULL.
- **/
+ */
 const char *
 _g_file_attribute_value_get_byte_string (const GFileAttributeValue *attr)
 {
@@ -487,16 +513,27 @@ _g_file_attribute_value_get_byte_string (const GFileAttributeValue *attr)
 
   return attr->u.string;
 }
-  
+
+char **
+_g_file_attribute_value_get_stringv (const GFileAttributeValue *attr)
+{
+  if (attr == NULL)
+    return NULL;
+
+  g_return_val_if_fail (attr->type == G_FILE_ATTRIBUTE_TYPE_STRINGV, NULL);
+
+  return attr->u.stringv;
+}
+
 /*
- * g_file_attribute_value_get_boolean:
+ * _g_file_attribute_value_get_boolean:
  * @attr: a #GFileAttributeValue.
- * 
+ *
  * Gets the boolean value from a file attribute value. If the value is not the
  * right type then %FALSE will be returned.
- * 
+ *
  * Returns: the boolean value contained within the attribute, or %FALSE.
- **/
+ */
 gboolean
 _g_file_attribute_value_get_boolean (const GFileAttributeValue *attr)
 {
@@ -507,16 +544,16 @@ _g_file_attribute_value_get_boolean (const GFileAttributeValue *attr)
 
   return attr->u.boolean;
 }
-  
+
 /*
- * g_file_attribute_value_get_uint32:
+ * _g_file_attribute_value_get_uint32:
  * @attr: a #GFileAttributeValue.
- * 
- * Gets the unsigned 32-bit integer from a file attribute value. If the value 
- * is not the right type then %0 will be returned.
- * 
- * Returns: the unsigned 32-bit integer from the attribute, or %0.
- **/
+ *
+ * Gets the unsigned 32-bit integer from a file attribute value. If the value
+ * is not the right type then 0 will be returned.
+ *
+ * Returns: the unsigned 32-bit integer from the attribute, or 0.
+ */
 guint32
 _g_file_attribute_value_get_uint32 (const GFileAttributeValue *attr)
 {
@@ -529,14 +566,14 @@ _g_file_attribute_value_get_uint32 (const GFileAttributeValue *attr)
 }
 
 /*
- * g_file_attribute_value_get_int32:
+ * _g_file_attribute_value_get_int32:
  * @attr: a #GFileAttributeValue.
- * 
- * Gets the signed 32-bit integer from a file attribute value. If the value 
- * is not the right type then %0 will be returned.
- * 
- * Returns: the signed 32-bit integer from the attribute, or %0.
- **/
+ *
+ * Gets the signed 32-bit integer from a file attribute value. If the value
+ * is not the right type then 0 will be returned.
+ *
+ * Returns: the signed 32-bit integer from the attribute, or 0.
+ */
 gint32
 _g_file_attribute_value_get_int32 (const GFileAttributeValue *attr)
 {
@@ -549,14 +586,14 @@ _g_file_attribute_value_get_int32 (const GFileAttributeValue *attr)
 }
 
 /*
- * g_file_attribute_value_get_uint64:
+ * _g_file_attribute_value_get_uint64:
  * @attr: a #GFileAttributeValue.
- * 
- * Gets the unsigned 64-bit integer from a file attribute value. If the value 
- * is not the right type then %0 will be returned.
- * 
- * Returns: the unsigned 64-bit integer from the attribute, or %0.
- **/  
+ *
+ * Gets the unsigned 64-bit integer from a file attribute value. If the value
+ * is not the right type then 0 will be returned.
+ *
+ * Returns: the unsigned 64-bit integer from the attribute, or 0.
+ */
 guint64
 _g_file_attribute_value_get_uint64 (const GFileAttributeValue *attr)
 {
@@ -569,14 +606,14 @@ _g_file_attribute_value_get_uint64 (const GFileAttributeValue *attr)
 }
 
 /*
- * g_file_attribute_value_get_int64:
+ * _g_file_attribute_value_get_int64:
  * @attr: a #GFileAttributeValue.
- * 
- * Gets the signed 64-bit integer from a file attribute value. If the value 
- * is not the right type then %0 will be returned.
- * 
- * Returns: the signed 64-bit integer from the attribute, or %0. 
- **/
+ *
+ * Gets the signed 64-bit integer from a file attribute value. If the value
+ * is not the right type then 0 will be returned.
+ *
+ * Returns: the signed 64-bit integer from the attribute, or 0.
+ */
 gint64
 _g_file_attribute_value_get_int64 (const GFileAttributeValue *attr)
 {
@@ -589,13 +626,13 @@ _g_file_attribute_value_get_int64 (const GFileAttributeValue *attr)
 }
 
 /*
- * g_file_attribute_value_get_object:
+ * _g_file_attribute_value_get_object:
  * @attr: a #GFileAttributeValue.
- * 
- * Gets the GObject from a file attribute value. If the value 
+ *
+ * Gets the GObject from a file attribute value. If the value
  * is not the right type then %NULL will be returned.
- * 
- * Returns: the GObject from the attribute, or %0.
+ *
+ * Returns: the GObject from the attribute, or %NULL.
  **/
 GObject *
 _g_file_attribute_value_get_object (const GFileAttributeValue *attr)
@@ -626,47 +663,57 @@ _g_file_attribute_value_set_from_pointer (GFileAttributeValue *value,
       else
 	value->u.string = value_p;
       break;
-      
+
+    case G_FILE_ATTRIBUTE_TYPE_STRINGV:
+      if (dup)
+	value->u.stringv = g_strdupv (value_p);
+      else
+	value->u.stringv = value_p;
+      break;
+
     case G_FILE_ATTRIBUTE_TYPE_OBJECT:
       if (dup)
 	value->u.obj = g_object_ref (value_p);
       else
 	value->u.obj = value_p;
       break;
-      
+
     case G_FILE_ATTRIBUTE_TYPE_BOOLEAN:
       value->u.boolean = *(gboolean *)value_p;
       break;
-      
+
     case G_FILE_ATTRIBUTE_TYPE_UINT32:
       value->u.uint32 = *(guint32 *)value_p;
       break;
-      
+
     case G_FILE_ATTRIBUTE_TYPE_INT32:
       value->u.int32 = *(gint32 *)value_p;
       break;
-      
+
     case G_FILE_ATTRIBUTE_TYPE_UINT64:
       value->u.uint64 = *(guint64 *)value_p;
       break;
-      
+
     case G_FILE_ATTRIBUTE_TYPE_INT64:
       value->u.int64 = *(gint64 *)value_p;
       break;
+
+    case G_FILE_ATTRIBUTE_TYPE_INVALID:
+      break;
+
     default:
       g_warning ("Unknown type specified in g_file_info_set_attribute\n");
       break;
     }
 }
-  
+
 /*
- * g_file_attribute_value_set_string:
+ * _g_file_attribute_value_set_string:
  * @attr: a #GFileAttributeValue.
  * @string: a string to set within the type.
- * 
+ *
  * Sets the attribute value to a given string.
- * 
- **/
+ */
 void
 _g_file_attribute_value_set_string (GFileAttributeValue *attr,
 				    const char          *string)
@@ -680,13 +727,12 @@ _g_file_attribute_value_set_string (GFileAttributeValue *attr,
 }
 
 /*
- * g_file_attribute_value_set_byte_string:
+ * _g_file_attribute_value_set_byte_string:
  * @attr: a #GFileAttributeValue.
  * @string: a byte string to set within the type.
- * 
+ *
  * Sets the attribute value to a given byte string.
- * 
- **/
+ */
 void
 _g_file_attribute_value_set_byte_string (GFileAttributeValue *attr,
 					 const char          *string)
@@ -699,14 +745,26 @@ _g_file_attribute_value_set_byte_string (GFileAttributeValue *attr,
   attr->u.string = g_strdup (string);
 }
 
+void
+_g_file_attribute_value_set_stringv (GFileAttributeValue *attr,
+				     char               **value)
+{
+  g_return_if_fail (attr != NULL);
+  g_return_if_fail (value != NULL);
+
+  _g_file_attribute_value_clear (attr);
+  attr->type = G_FILE_ATTRIBUTE_TYPE_STRINGV;
+  attr->u.stringv = g_strdupv (value);
+}
+
+
 /*
- * g_file_attribute_value_set_boolean:
+ * _g_file_attribute_value_set_boolean:
  * @attr: a #GFileAttributeValue.
  * @value: a #gboolean to set within the type.
- * 
- * Sets the attribute value to the given boolean value. 
- * 
- **/
+ *
+ * Sets the attribute value to the given boolean value.
+ */
 void
 _g_file_attribute_value_set_boolean (GFileAttributeValue *attr,
 				     gboolean             value)
@@ -719,32 +777,30 @@ _g_file_attribute_value_set_boolean (GFileAttributeValue *attr,
 }
 
 /*
- * g_file_attribute_value_set_uint32:
+ * _g_file_attribute_value_set_uint32:
  * @attr: a #GFileAttributeValue.
  * @value: a #guint32 to set within the type.
- * 
+ *
  * Sets the attribute value to the given unsigned 32-bit integer.
- * 
- **/ 
+ */
 void
 _g_file_attribute_value_set_uint32 (GFileAttributeValue *attr,
 				    guint32              value)
 {
   g_return_if_fail (attr != NULL);
-  
+
   _g_file_attribute_value_clear (attr);
   attr->type = G_FILE_ATTRIBUTE_TYPE_UINT32;
   attr->u.uint32 = value;
 }
 
 /*
- * g_file_attribute_value_set_int32:
+ * _g_file_attribute_value_set_int32:
  * @attr: a #GFileAttributeValue.
  * @value: a #gint32 to set within the type.
- * 
+ *
  * Sets the attribute value to the given signed 32-bit integer.
- *  
- **/
+ */
 void
 _g_file_attribute_value_set_int32 (GFileAttributeValue *attr,
 				   gint32               value)
@@ -757,13 +813,12 @@ _g_file_attribute_value_set_int32 (GFileAttributeValue *attr,
 }
 
 /*
- * g_file_attribute_value_set_uint64:
+ * _g_file_attribute_value_set_uint64:
  * @attr: a #GFileAttributeValue.
  * @value: a #guint64 to set within the type.
- * 
+ *
  * Sets the attribute value to a given unsigned 64-bit integer.
- * 
- **/
+ */
 void
 _g_file_attribute_value_set_uint64 (GFileAttributeValue *attr,
 				    guint64              value)
@@ -776,13 +831,12 @@ _g_file_attribute_value_set_uint64 (GFileAttributeValue *attr,
 }
 
 /*
- * g_file_attribute_value_set_int64:
+ * _g_file_attribute_value_set_int64:
  * @attr: a #GFileAttributeValue.
  * @value: a #gint64 to set within the type.
- * 
- * Sets the attribute value to a given signed 64-bit integer. 
- * 
- **/
+ *
+ * Sets the attribute value to a given signed 64-bit integer.
+ */
 void
 _g_file_attribute_value_set_int64 (GFileAttributeValue *attr,
 				   gint64               value)
@@ -795,14 +849,13 @@ _g_file_attribute_value_set_int64 (GFileAttributeValue *attr,
 }
 
 /*
- * g_file_attribute_value_set_object:
+ * _g_file_attribute_value_set_object:
  * @attr: a #GFileAttributeValue.
  * @obj: a #GObject.
  *
  * Sets the attribute to contain the value @obj.
  * The @attr references the GObject internally.
- * 
- **/
+ */
 void
 _g_file_attribute_value_set_object (GFileAttributeValue *attr,
 				    GObject             *obj)
@@ -830,40 +883,40 @@ list_update_public (GFileAttributeInfoListPriv *priv)
 
 /**
  * g_file_attribute_info_list_new:
- * 
+ *
  * Creates a new file attribute info list.
- * 
+ *
  * Returns: a #GFileAttributeInfoList.
- **/
+ */
 GFileAttributeInfoList *
 g_file_attribute_info_list_new (void)
 {
   GFileAttributeInfoListPriv *priv;
 
   priv = g_new0 (GFileAttributeInfoListPriv, 1);
-  
+
   priv->ref_count = 1;
   priv->array = g_array_new (TRUE, FALSE, sizeof (GFileAttributeInfo));
-  
+
   list_update_public (priv);
-  
+
   return (GFileAttributeInfoList *)priv;
 }
 
 /**
  * g_file_attribute_info_list_dup:
  * @list: a #GFileAttributeInfoList to duplicate.
- * 
+ *
  * Makes a duplicate of a file attribute info list.
- * 
- * Returns: a copy of the given @list. 
- **/
+ *
+ * Returns: a copy of the given @list.
+ */
 GFileAttributeInfoList *
 g_file_attribute_info_list_dup (GFileAttributeInfoList *list)
 {
   GFileAttributeInfoListPriv *new;
   int i;
-  
+
   g_return_val_if_fail (list != NULL, NULL);
 
   new = g_new0 (GFileAttributeInfoListPriv, 1);
@@ -878,47 +931,47 @@ g_file_attribute_info_list_dup (GFileAttributeInfoList *list)
       new->public.infos[i].type = list->infos[i].type;
       new->public.infos[i].flags = list->infos[i].flags;
     }
-  
+
   return (GFileAttributeInfoList *)new;
 }
 
 /**
  * g_file_attribute_info_list_ref:
  * @list: a #GFileAttributeInfoList to reference.
- * 
+ *
  * References a file attribute info list.
- * 
+ *
  * Returns: #GFileAttributeInfoList or %NULL on error.
- **/
+ */
 GFileAttributeInfoList *
 g_file_attribute_info_list_ref (GFileAttributeInfoList *list)
 {
   GFileAttributeInfoListPriv *priv = (GFileAttributeInfoListPriv *)list;
-  
+
   g_return_val_if_fail (list != NULL, NULL);
   g_return_val_if_fail (priv->ref_count > 0, NULL);
-  
+
   g_atomic_int_inc (&priv->ref_count);
-  
+
   return list;
 }
 
 /**
  * g_file_attribute_info_list_unref:
  * @list: The #GFileAttributeInfoList to unreference.
- * 
+ *
  * Removes a reference from the given @list. If the reference count
  * falls to zero, the @list is deleted.
- **/
+ */
 void
 g_file_attribute_info_list_unref (GFileAttributeInfoList *list)
 {
   GFileAttributeInfoListPriv *priv = (GFileAttributeInfoListPriv *)list;
   int i;
-  
+
   g_return_if_fail (list != NULL);
   g_return_if_fail (priv->ref_count > 0);
-  
+
   if (g_atomic_int_dec_and_test (&priv->ref_count))
     {
       for (i = 0; i < list->n_infos; i++)
@@ -932,7 +985,7 @@ g_file_attribute_info_list_bsearch (GFileAttributeInfoList *list,
 				    const char             *name)
 {
   int start, end, mid;
-  
+
   start = 0;
   end = list->n_infos;
 
@@ -954,26 +1007,26 @@ g_file_attribute_info_list_bsearch (GFileAttributeInfoList *list,
  * g_file_attribute_info_list_lookup:
  * @list: a #GFileAttributeInfoList.
  * @name: the name of the attribute to lookup.
- * 
+ *
  * Gets the file attribute with the name @name from @list.
  *
- * Returns: a #GFileAttributeInfo for the @name, or %NULL if an 
+ * Returns: a #GFileAttributeInfo for the @name, or %NULL if an
  * attribute isn't found.
- **/
+ */
 const GFileAttributeInfo *
 g_file_attribute_info_list_lookup (GFileAttributeInfoList *list,
 				   const char             *name)
 {
   int i;
-  
+
   g_return_val_if_fail (list != NULL, NULL);
   g_return_val_if_fail (name != NULL, NULL);
-  
+
   i = g_file_attribute_info_list_bsearch (list, name);
 
   if (i < list->n_infos && strcmp (list->infos[i].name, name) == 0)
     return &list->infos[i];
-  
+
   return NULL;
 }
 
@@ -983,20 +1036,20 @@ g_file_attribute_info_list_lookup (GFileAttributeInfoList *list,
  * @name: the name of the attribute to add.
  * @type: the #GFileAttributeType for the attribute.
  * @flags: #GFileAttributeInfoFlags for the attribute.
- * 
+ *
  * Adds a new attribute with @name to the @list, setting
- * its @type and @flags. 
- **/
+ * its @type and @flags.
+ */
 void
-g_file_attribute_info_list_add    (GFileAttributeInfoList *list,
-				   const char             *name,
-				   GFileAttributeType      type,
-				   GFileAttributeInfoFlags flags)
+g_file_attribute_info_list_add (GFileAttributeInfoList *list,
+				const char             *name,
+				GFileAttributeType      type,
+				GFileAttributeInfoFlags flags)
 {
   GFileAttributeInfoListPriv *priv = (GFileAttributeInfoListPriv *)list;
   GFileAttributeInfo info;
   int i;
-  
+
   g_return_if_fail (list != NULL);
   g_return_if_fail (name != NULL);
 
